#ifndef __DOWNLOAD_H__
#define __DOWNLOAD_H__

#include <glib-object.h>
#include <libxml/uri.h>

#include "linelog.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef enum {
    DOWNLOAD_QUEUE_NONE = 0,
    DOWNLOAD_QUEUE_ACTIVE,
    DOWNLOAD_QUEUE_BATCH,
    DOWNLOAD_QUEUE_STOPPED,
    DOWNLOAD_QUEUE_DONE
} DownloadQueue;

typedef enum {
    DOWNLOAD_EXIT_NONE = 0,
    DOWNLOAD_EXIT_SHUTDOWN,
    DOWNLOAD_EXIT_STOPPED,
    DOWNLOAD_EXIT_QUEUED,
    DOWNLOAD_EXIT_DELETE,
    DOWNLOAD_EXIT_DONE
} DownloadExitCause;

typedef enum {
    DOWNLOAD_STOP_NONE,
    DOWNLOAD_STOP_SHUTDOWN,
    DOWNLOAD_STOP_DELETE,
    DOWNLOAD_STOP_QUEUE
} DownloadStopAction;

#define DOWNLOAD(obj)           G_TYPE_CHECK_INSTANCE_CAST (obj, download_get_type (), Download)
#define DOWNLOAD_CLASS(klass)   G_TYPE_CHECK_CLASS_CAST (klass, download_get_type (), DownloadClass)
#define IS_DOWNLOAD(obj)        G_TYPE_CHECK_INSTANCE_TYPE (obj, download_get_type ())


typedef struct _Download       Download;
typedef struct _DownloadClass  DownloadClass;

struct _Download
{
    GObject object;

    /*< private >*/

    guint id;
    gchar *url_string;
    xmlURIPtr url;

    gboolean default_path;
    gchar *local_path;

    GStaticMutex download_mutex;

    gulong bytes_done;
    gulong bytes_total;

    guint speed;

    List *threads;

    struct sockaddr_in *addr;
    struct timeval resolve_time;

    gint target_ready;
    DownloadQueue queue;
//FIXME: convert enum to type
    enum dm_download_status status;
    DownloadExitCause exit_cause;

    /* loggin facilities */
    Linelog *linelog;

    /* settings */
    guint recursive;
    /* the limit set on the download */
    guint speedlimit;
    /* the limit the download must obey (perhaps project mandated) */
    guint real_speedlimit;
};

struct _DownloadClass
{
    GObjectClass parent_class;

    void (* download_update_signal)     (void);
    void (* download_exit_signal)       (void);
};

GType           download_get_type       (void);
Download*       download_new            (gchar *url);

void    download_start      (Download *download, GError **error);
void    download_stop       (Download *download,
                             DownloadStopAction action);
void    download_delete     (Download *download);

void    download_url_set    (Download *download, gchar *url);

void    download_path_set   (Download *download, gchar *path);

gint    download_status_get (Download *download);
void    download_status_set (Download *download,
                             enum dm_download_status status);

guint   download_recursive_get          (Download *download);
void    download_recursive_set          (Download *download, guint recursive);
void    download_default_path_set       (Download *download, gboolean setting);
void    download_speedlimit_set         (Download *download, guint speed);
guint   download_speedlimit_get         (Download *download);
void    download_real_speedlimit_set    (Download *download, guint speed);
guint   download_real_speedlimit_get    (Download *download);

DownloadExitCause   download_exit_cause (Download *download);

guint   download_id_get     (Download *download);
void    download_id_set     (Download *download, guint id);

const gchar *       download_url_string (Download *download);

gulong  download_bytes_done_get     (Download *download);
void    download_bytes_done_set     (Download *download, gulong bytes_done);
gulong  download_bytes_total_get    (Download *download);
void    download_bytes_total_set    (Download *download, gulong bytes_total);
guint   download_speed              (Download *download);


DownloadQueue   download_queue_get  (Download *download);
void            download_queue_set  (Download *download,
                                     DownloadQueue queue);

void            download_logdir_set (Download *download, gchar *path);
void            download_log_add    (Download *download, const gchar *format,
                                     ...);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __DOWNLOAD_H__ */
